;Mouse demo in TASM-like syntax

.binarymode ti8x

#include "usb8xtsm.inc"
#include "ti83plus.inc"
#include "mirage.inc"

USBDriverBuf = appbackupscreen		;Memory for driver 
USBDriverCall = appbackupscreen+128		;Memory for U_CALL system
DBuf = appbackupscreen+256			;Temporary memory for descriptors

MouseX EQU appbackupscreen+512		;X coordinate of mouse cursor
MouseY EQU appbackupscreen+513		;Y coordinate of mouse cursor
MouseBtn EQU appbackupscreen+514		;button state of mouse

	.org 9D93h
	.db	0BBh,06Dh

ProgStart:
	U_CALL_INIT(Oops)			;Initialize U_CALL system
	jp	c,error

	U_CALL(GetVersion)
	ld	de,0Bh
	or	a
	sbc	hl,de
	jp	c,error			;Requires version 0.11+

	ld	hl,USBDriverBuf
      U_CALL(DriverInit)		;Initialize Driver
	jp	c,error

	ld	hl,DBuf
	ld	bc,256
	bcall(_MemClear)			;Clear descriptor buffer

	ld	de,DBuf
	ld	b,0				;allow diagonal keys
	U_CALL(MouseInit)		;Initialize Mouse
	jp	c,error

	bcall(_GrBufClr)

	xor	a
	ld	(MouseBtn),a
	ld	a,50*2
	ld	(MouseX),a
	ld	a,32*2
	ld	(MouseY),a
	call	DispCursor

MainLoop:
	bcall(_GrBufCpy)
	bcall(_GetCSC)
	cp	skClear
	jr	z,MouseDone
	call	DispCursor			;Erase cursor
	ld	b,4				;Buffer to screen copy is kind of slow, so let's 
gkloop:					; update the cursor a few times for each buffer copy
	push	bc
	U_CALL(MouseGetKey)		;Get mouse info from usb8x
	call	MouseUpdateCursor		;Update cursor position
	pop	bc
	djnz	gkloop
	call	DispCursor			;Re-display cursor
	jr	MainLoop

MouseDone:
	U_CALL(HostKill)			;Disconnect power from mouse
	U_CALL(DriverKill)		;Kill driver
	ret



MouseUpdateCursor:
;Button presses are in B
;Directional information is in C
       ld     a,b
       ld     (MouseBtn),a
       ld     hl,MouseX
       bit    mouseBitRight,c
       jr     z,munoright
       inc    (hl)
munoright:
       bit    mouseBitLeft,c
       jr     z,munoleft
       dec    (hl)
munoleft:
       ld     hl,MouseY
       bit    mouseBitDown,c
       jr     z,munodown
       inc    (hl)
munodown:
       bit    mouseBitUp,c
       ret    z
       dec    (hl)
       ret


DispCursor:
       ld     ix,MouseSpr
       ld     a,(MouseBtn)
       or     a
       jr     z,dcreg
       ld     ix,SmileSpr
dcreg:
       ld     a,(MouseY)
       srl    a
       ld     l,a
       ld     a,(MouseX)
       srl    a
       call   PutSprite
       ret

SmileSpr:
       .db     01100110b
       .db     01100110b
       .db     00000000b
       .db     00011000b
       .db     10000001b
       .db     01100110b
       .db     00111100b
       .db     00000000b

MouseSpr:
       .db     11111000b
       .db     11110000b
       .db     11110000b
       .db     11001000b
       .db     10000100b
       .db     00000010b
       .db     00000001b
       .db     00000000b

Oops:			
;This will never get called.  Mouse code sets its own callback
	ret

Error:
;Display error code and exit
	ld	a,'E'
	bcall(_putc)
	U_CALL(GetErrorCode)
	push	bc
	pop	hl
	bcall(_disphl)
	bcall(_getkey)
	ret

PutSprite:
;ix -> sprite
;b = height
;a = x coord
;l = y coord
       ld     b,8
       ld     d,0
       ld     e,l
       ld     h,d
       add    hl,hl
       add    hl,hl
       add    hl,de
       add    hl,de
       add    hl,hl         ;y coord *12 offset to first row
       ld     e,a
       srl    e
       srl    e
       srl    e             ;x coord byte offset
       add    hl,de         ;offset to start of byte
       ld     de,plotSScreen
       add    hl,de
       and    7             ;number of bits to rotate
       ld     c,a
PutSpriteLoop:
       ld     a,c           ;bits to shift
       ld     d,0           ;2nd byte to load
       ld     e,(ix)        ;1st byte to load
       or     a
       jr     z,NoShift
psloop2:
       srl    e
       rr     d
       dec    a
       jr     nz,psloop2
NoShift:
       ld     a,(hl)
       xor    e
       ld     (hl),a
       inc    hl
       ld     a,(hl)
       xor    d
       ld     (hl),a
       ld     de,11
       add    hl,de
       inc    ix
       djnz   PutSpriteLoop
       ret